# -*- coding:utf-8 -*-
# !/usr/bin/env python 3.9.11
"""
@File    :  enums.py
@Time    :  2022/5/2 2:52 AM
@Author  :  YuYanQing
@Version :  1.0
@Contact :  mryu168@163.com
@License :  (C)Copyright 2022-2026
@Desc    :  None
"""


class GlobalVarEnum:
    APP_NAME = "custard"
    BEYOND_RESTRICT_MSG = "The interview is too fast, please have a cup of tea and take a break!"
    INIT_ERR_MSG = "You must call Limiter.init in startup event of fastapi!"
